pro simage_new_shapelets_decomp, f, beta, n_max, decomp, recomp, $
		      x0=x0, 		              $
                      noise=noise, 		      $
		      over=over,		      $
		      ls=ls,		              $
		      integrate=integrate,            $
		      sky=sky, 		              $
		      non1=non1, 		      $
		      psf=psf, 		              $
		      cov=cov, 		              $
		      silent=silent, 		      $
		      name=name

;$Id: shapelets_decomp.pro, v2$
;
; Copyright ? 2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
;+
; NAME:
;      SHAPELETS_DECOMP
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Decompose a function f into phi_n basis functions and
;      output the coefficients in an IDL structure.
;
; INPUTS:
;      f     - array containing f(x1,x2)
;      n_max - maximum order of the basis coefficients to decompose into
;      beta  - basis function scale length (beta)
;
; OPTIONAL INPUTS:
;      name  - Name of object (string)
;      x0    - Coordinates of the centre of the basis functions
;              DEFAULT: centre of the image
;      noise - inverse variance map of pixels. Assumes zero covariance.
;              if only one value, assumes this is rms on all pixls
;              DEFAULT: constant noise and chisq not normalised
;      psf   - a Cartesian decomp structure representing the local PSF
;      over  - Force an oversampling factor to evaluate
;              the basis functions (over=1: no oversampling;
;              DEFAULT: over=round(1/th_min) with th_min=w/sqrt(nmax)
;
; KEYWORD PARAMETERS:
;      /LS   - Fit coefficients with least-squares method instead of
;              Fourier-style overlap integrals
;              DEFAULT: least-squares (for overlap integral, set ls=2)
;      /COV  - Return full covariance matrix of coefficients in "error"
;              DEFAULT: return error on coefficients in error variable
;      /INT  - Use basis functions integrated within pixels instead of
;              central pixel value
;              DEFAULT: integration (to get central value, set int=2)
;      /SKY  - 1: fit sky background with a constant value
;              2: fit sky gradient with a plane
;              DEFAULT: no sky subtraction
;              NB: only works with LS fitting and this will make
;                  chisq different from that calculated again
;                  later using shapelets_recomp.
;      /NON1 - a_01 and a_10 are required to be zero, Kuijken-like
;
; OUTPUTS:
;      decomp structure: w, n_max, x0, nf, n1, n2, n_coeffs,
;                        a (set of coefficients),
;                        error (SDs or covariance matrix - see /cov above),
;                        over (how much it has been oversampled),
;                        int (whether integrated basis functions have been used),
;                        chisq ((0)total and (1)reduced),
;                        skyfit (background level and coefs of gradient)
;      recomp: recomposed image from the shapelets (still convolved
;              with the original PSF - to remove this, use shapelets_recomp)
;
; MODIFICATION HISTORY:
;      Jul 05 - External decomp creation routine incorporated by RM.
;      Jan 05 - RM optimised gamma and n_max_gamma during PSF deconvolution.
;      Apr 04 - RM added ability to mask pixels (by setting weight map to zero)
;               but still calculate the correct number of DOFs in the fit
;      Sep 03 - RM changed name of routine to shapelet_decomp.pro
;      Jul 03 - RM added "name" element to decomp structure
;      Apr 02 - RM added PSF deconvolution
;      Mar 02 - AR computed Chi^2 and fixed normalisation. Also added
;               option to return the recomposed (still convolved) image
;      Dec 01 - RM implemented least squares fitting of basis fns to data
;      Nov 01 - RM added background subtraction and sky fitting
;      Nov 01 - Richard Massey added numerical integration of basis fns
;      Feb 01 - AR incorporated oversampling of the basis functions
;      Jul 00 - AR changed to order by n=n1+n2
;      Jul 99 - mk_decomp.pro written by Alexandre Refregier
;-

COMPILE_OPT idl2
ON_ERROR,2

; Set defaults
if not keyword_set(integrate) then integrate=1   ; Could do better with: if n_elements(in) eq 0 then ...
if not keyword_set(name) then name=""            ; Create a blank string for the name of the object
if not keyword_set(ls) then ls=1                 ; Default is to fit rather than do overlap integrals
if ls ne 1 then sky=0                            ; Can't fit the sky if performing overlap integrals
if not keyword_set(noise) then noise=1.          ; Default is to assume constant noise
if n_max lt 2 then non1=0                        ; Can't avoid n=1 coeffs if there are no higher ones available!

; Determine the size of the postage stamp
fsize = size(f)	; Size of image array
nf1   = fsize[1]
nf2   = fsize[2]
n_pix = nf1*nf2

; If it has not been specified, set the centre of the basis functions to be
;   the centre of the postage stamp
if not keyword_set(x0) then x0=[nf1,nf2]/2.

; set amount of oversampling
if integrate eq 1 then over=1 $  ; don`t need to oversample if integrating!
else begin
  if not keyword_set(over) then begin
    th_min=beta/sqrt(n_max)
    over=round(1./th_min) ; oversample if necessary
  endif
endelse
if over le 1 then begin   ; no oversampling is necessary - set parameters
  over=1
  f_o=f                   ; subscript _o's mean "oversampled version"
  nf1_o=nf1
  nf2_o=nf2
  n_pix_o=nf1_o*nf2_o
  x0_o=x0
  beta_o=beta
endif else begin          ; oversampling is necessary - set alternate params
  over=round(over)
  nf1_o=nf1*over
  nf2_o=nf2*over
  n_pix_o=nf1_o*nf2_o
  f_o=rebin(f,nf1_o,nf2_o,/sample)
  x0_o=x0*float(over)
  beta_o=beta*float(over)
endelse

; reformat data and noise (inverse variance) maps into a vector
data_o = reform(f_o, n_pix_o, 1, /overwrite)
if ls eq 1 then begin
  
  print,'creating'
  
  V=fltarr(n_pix_o,n_pix_o)
  
  print,'populating'
  
  
  if (n_elements(noise) eq 1) then begin
    message,"WARNING: Artificial noise map being constructed",/info,noprint=silent
    ; all pixels will be weighted equally. If this is not the rms value
    ; on a pixel, Chi^2 will not be properly normalised!

;    noise_o = fltarr(nf1_o*nf2_o,/nozero) ; will be a (sparse) diagonal matrix
;    noise_o[*] = 1./noise^2  ; rms, and other option is INVERSE variance
;    BUG! should have said
;    noise_o[*] = noise    ; so that noise_o is indeed the inverse variance
  
    for i=0,n_pix_o-1 do V[i,i]=1./noise
    ; MatrixV is the convariance matrix between pixels. Diagonal elements are
    ; the variance, as in V, in Lupton p.84.
  
  endif else if (n_elements(noise) gt 1) then begin
;    noise_o = noise
;    noise_o = reform(rebin(noise_o,nf1_o,nf2_o,/sample), n_pix_o, 1, /overwrite)

    for i=0,n_pix_o-1 do V[i,i]=1./noise[i]

  endif

  print,'making sparse'
  sparse_v = sprsin(V) 
  print,'done'

endif
shapelets_make_nvec, n_max, n1, n2, n_coeffs

; convolve basis functions with psf
if keyword_set(psf) then begin
  alpha=beta_o
  n_max_a=n_max
  ; Leave it up to subroutines to decide values for gamma and n_max_gamm  ;n_max_gamma=(n_max_a>psf.n_max)+4
  P_nm=simage_shapelets_convolution_matrix(psf,alpha,n_max_a,gamma,n_max_gamma)

  ; Create least-squares matrix of function values
  resolve_routine,"shapelets_make_ls_matrix",/COMPILE_FULL_FILE,/NO_RECOMPILE
  simage_shapelets_make_ls_matrix, MatrixT, n_max_gamma, gamma, nf1_o, nf2_o, x0_o, $
     sky=sky, integrate=integrate, non1=0
  
  ; Add in extra dimensions for sky fitting and remove them to supress n=1 fitting
  if keyword_set(sky) then simage_shapelets_make_ls_matrix,P_nm,sky=sky,/extend_zeros
  if keyword_set(non1) and ls eq 1 then simage_shapelets_make_ls_matrix,P_nm,/trans,/remove_zeros
  ; Old version of this, using subroutines hidden in another file
  ;if keyword_set(sky) then simage_shapelets_extend_zeros,P_nm,sky
  ;if keyword_set(non1) and ls eq 1 then simage_shapelets_remove_zeros,P_nm,/trans

endif else begin
   shapelets_make_ls_matrix, MatrixT, n_max, beta_o, nf1_o, nf2_o, x0_o, $
     sky=sky, integrate=integrate, non1=non1
endelse


; actual decomposition bit
if ls eq 1 then begin

  if keyword_set(psf) then MatrixT=transpose(P_nm)#MatrixT

  ; perform least-squares linear algebra fit
  ; (Matrix is transpose(M) of Anton p460+ or M in Lupton p84)

  Matrix=transpose(MatrixT)
  help,MatrixT,V
  help,sparse_V,/st
  print,'inverting'
;  stop

n_pix_o=6
V=V[0:5,0:5]
sparse_v = sprsin(V) 

  for i=0,n_coeffs-1 do begin
  
    est=fltarr(n_pix_o)
    est[i]=1.
    vec=reform(MatrixT[i,0:5])
    help,vec,est
    x=linbcg(sparse_V, vec,est)
  
    help,x
  
  endfor

;  MatrixVinv=invert(MatrixV)
  stop
  if ( keyword_set(noise) ) then $
    for i=0L,n_pix_o-1 do MatrixT[*,i]=MatrixT[*,i]*noise_o[i] ; / since V^-1 / since INVERSE variance
  ; NB: this might be more numerically stable using SVD (cf Berry, Hobson & Withington 2004)
  MatrixI=invert(MatrixT#Matrix) ; size of matrix to invert is only n_coeffs^2
  coeffs=MatrixI#MatrixT#data_o

endif else begin
  ; perform normal linear "overlap integral" type of decomposition
  coeffs=MatrixT#data_o
  error=fltarr(n_coeffs) ; can`t calculate error on coeffs so just set to zero
  Matrix=transpose(MatrixT)

  if keyword_set(psf) then coeffs=invert(P_nm)#coeffs  ; IS THIS CORRECT?
  if keyword_set(noise) then message, $
    "WARNING: Noise properties not used in linear decomposition",/info,noprint=silent
endelse



; re-insert a_01 and a_10 if they had been left out
if keyword_set(non1) then begin
  if ls eq 1 then begin
    coeffs=[coeffs[0],0.,0.,coeffs[1:*]]
   simage_ shapelets_make_ls_matrix,MatrixI,matrix2=Matrix,/add_zeros
    ; Old version of this, using subroutines hidden in another file
    ;shapelets_add_zeros,MatrixI,Matrix
  endif else begin
    coeffs[1]=0. & coeffs[2]=0.
  endelse
endif


; make the recomposed (but still PSF convolved and including sky) image
recomp_o=Matrix#coeffs
recomp=rebin(reform(recomp_o,nf1_o,nf2_o),nf1,nf2)


; discard sky background fit if it had been done
if keyword_set(sky) then begin
  skyfit=coeffs[n_coeffs:*]
  while n_elements(skyfit) lt 3 do skyfit=[skyfit,0.]
  coeffs=coeffs[0:n_coeffs-1]
  MatrixI=MatrixI[0:n_coeffs-1,0:n_coeffs-1]
  Matrix=Matrix[*,0:n_coeffs-1]
endif else skyfit=fltarr(3)

; calculate the covariance matrix of coefficients
if ls eq 1 then begin
  if keyword_set(cov) then coeffs_error=MatrixI $
  else begin
    coeffs_error=fltarr(n_coeffs)
    for i=0,n_coeffs-1 do coeffs_error[i]=sqrt(MatrixI[i,i])
  endelse
endif else coeffs_error=fltarr(n_coeffs)



; compute Chi^2 (directly from the original image and the recomposed
; version - exactly the same calculation as that with matrices in Lupton)
chisq=total((data_o-recomp_o)^2*noise_o)
masked_pixels=where(noise_o eq 0.,n_masked_pixels)
dof=n_pix-n_masked_pixels-n_coeffs
if keyword_set(sky) then dof=dof-sky*(sky+1)/2
if keyword_set(non1) then dof=dof+2
chisq_reduced=chisq/float(dof)
chisq=[chisq,chisq_reduced]



; warn if the number of coefficients is larger than the number of pixels
if dof le 0 then $
  message,"WARNING: No of coefficients "+strtrim(n_pix-dof,1)+ $
          " > No of pixels "+strtrim(n_pix,1),/info,noprint=silent
  ; Could prevent least-squares from overfitting and then failing
  ; by checking this earlier? Then should set n_max to a lower number,
  ; but store n_max and recover it afterwards, setting a(^) to zero.


; store in a structure
decomp=simage_shapelets_create_decomp(n_max)
decomp.name=name
decomp.x=x0
decomp.beta=beta
decomp.coeffs=coeffs
decomp.coeffs_error=coeffs_error
decomp.n_pixels=[nf1,nf2]
decomp.over=over
decomp.integrate=integrate
decomp.chisq=chisq
decomp.sky_level=skyfit[0]
decomp.sky_slope=skyfit[1:2]

end

