pro simage_sex_2, filename, filename_detect=filename_detect,SEXCAT=sexcat,                     $
                   SILENT=silent,                     $
                   FULL_PATH=full_path,               $
                   ASCII=ascii,                       $
                   TELESCOPE=telescope,               $
                   SEEING=seeing,                     $
                   FILTER=filter,                     $
                   BACKGROUND_LEVEL=background_level, $
                   RMS_BACKGROUND=rms_background,     $
                   SEGMENTATION=segmentation,         $
                   detect_minarea=detect_minarea,     $
                   detect_thresh=detect_thresh,       $
                   analysis_thresh=analysis_thresh,   $
                   deblend_nthresh=deblend_nthresh,   $
                   deblend_mincont=deblend_mincont,   $
                   gain=gain,                         $
                   memory_objstack=memory_objstack,   $
                   memory_pixstack=memory_pixstack,   $
                   memory_bufsize=memory_bufsize     
;MF - added ability to change various parameters, line 202 - 7/07/05
;MF - dual image mode: line 160
;MF - default telescope is UDF, line 124
;MF - dual image weighting: line 199
;$Id: sex.pro, v2$
;
; Copyright ? 2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
;+
; NAME:
;       SEX
;
; PURPOSE:
;       Runs SExtractor (by spawning it as a child process under the shell
;       used to run IDL) to locate objects within an image.
;
; CATEGORY:
;       ROI finder.
;
; CALLING PROCEDURE:
;       sex,"example"
;
; INPUTS:
;       FILENAME - Name of input image
;
; OPTIONAL INPUTS:
;       TELESCOPE - Gets SExtractor parameters right. Default is HDF.
;       FILTER    - SExtractor convolution filter. Default is "gauss_2.0_5x5"
;       SEEING    - Used for star/galaxy separation.
;
; KEYWORD PARAMETERS:
;       /SILENT   - Operates silently.
;       /ASCII    - Writes out an ASCII catalogue, rather than a binary (fits)
;                   table. There is a similar switch in
;                   shapelets_read_sexcat.pro to read in the two file types.
;       /SEGMENTA - Also output a segmentation map.
;       /BACKGROU - Also ouptut an image containing the local background level.
;       /RMS_BACK - Also ouptut an image containing the local noise rms.
;       /FULL_PAT - Assume the input filename is absolute, rather than relative
;                   to some location specified in shapelets_paths.pro.
;
; OUTPUTS:
;       SExtractor is run and outputs various files to disc.
;
; OPTIONAL OUTPUTS:
;       SEXCAT    - SExtractor catalogue data structure, automatically read
;                   back in from disc.
;
; TO DO:
;       Try a two-stage version of SExtractor. Run once to get rough catalogue
;       of stars. Then run again with a new seeing setting. Bertin's rule of
;       thumb for setting the seeing in default.sex file is SEEING=FLUX_RADIUS*2
;       This should improve star/galaxy separation in SExtractor, at a minimal
;       cost incomputing time.
;
;       Add information (to fits headers? to a new file format?) storing the
;       parameter file, the date, the image name, etc. for future reference.
;
; PROCEDURES USED:
;       None. But requires SExtractor to be installed on the computer, with
;       the full path of the "sex" executable stored in shapelets_paths.pro.
;
; MODIFICATION HISTORY:
;       Dec 05 - Bug in the full_path option fixed by RM
;       Oct 05 - Spurious UNIX commands removed by RM and replaced with IDL code
;       Oct 05 - Catalogue written to intended folder by Joel Berge
;       Apr 05 - RMS keyword added by Will high and RM
;       Apr 05 - SEXCAT keyword added by RM
;       Mar 05 - SEEING keyword added by RM
;       Mar 05 - Can change columns in SExtractor fits (but not ASCII) output by RM
;       Jan 05 - Appropriate message displayed after errors spawning sex by RM
;       Jan 05 - Option to write out ASCII/FITS format catalogues added by RM
;       Jul 04 - The segmentation file is now called _seg.fits by Alexandre Refregier
;       May 04 - Convention that weight maps stored as _weight.fits adopted by RM
;       Sep 03 - sex.pro adapted from shapelets_read_sexcat.pro by RM
;       May 03 - Ellipticity e,e1,e2 notation added to structure by RM
;       Mar 03 - Filepaths generalised and routine renamed shapelets_read_sexcat by RM
;       Apr 02 - rd_scat.pro adapted from shapelets_extractor.pro by RM
;       Dec 01 - First version of shapelets_extractor.pro written by Richard Massey
;-

COMPILE_OPT idl2
ON_ERROR,2


; Set defaults
if not keyword_set(silent)    then silent=0              ; Operate silently?
if not keyword_set(telescope) then telescope="UDF"	 ; Gets SEx params right ;MF - default is UDF
if not keyword_set(filter)    then filter="gauss_2.0_5x5"; SExtractor detection filter
if not keyword_set(filename)  then message,"You must specify a file name!"
if keyword_set(full_path) then begin
  ; Strip file extension
  input_file=strmid(filename,0,strpos(strlowcase(filename),".fit"))
  if keyword_set(filename_detect) then input_file_detect = strmid(filename_detect,0,strpos(strlowcase(filename_detect),".fit")) ;MF - 6/6/06
  output_file=strmid(filename,0,strpos(strlowcase(filename),".fit"))
endif else begin
  ; Add relevant path name
  input_file=simage_shapelets_paths(3,silent=silent)+filename
  if keyword_set(filename_detect) then input_file_detect = simage_shapelets_paths(3,silent=silent)+filename_detect
  output_file=simage_shapelets_paths(2,silent=silent)+filename
endelse


; Check to see if the image file exists
if ( not file_test(input_file+".fits") ) then $
	message,"Image "+input_file+".fits not available!"
if keyword_set(filename_detect)  then if not file_test(input_file_detect+'.fits') then $
  message,'Image '+input_file_detect+'.fits not available!'

; Print intent to screen
message,"Locating objects with SExtractor",/info,noprint=silent


; Decide format and file extension for backwards compatability
if keyword_set(ASCII) then begin
  format="ASCII_HEAD"
  extension="sex"
  paramfile="shex_ascii.param"
endif else begin
  format="FITS_LDAC"  
  extension="sexcat"
  paramfile="shex.param"
endelse


; Form shell command
sex_dir=simage_shapelets_paths(7,/silent)
if not keyword_set(input_file_detect) then input_file_detect = ' ' else input_file_detect=' '+input_file_detect+'.fits, '  ;MF - added in this file_detect part
command_line=simage_shapelets_paths(8,/silent)+input_file_detect+input_file+".fits"     +$ ;MF - 6/6/06
  " -c "              +sex_dir+telescope+".sex"              +$
  " -STARNNW_NAME "   +sex_dir+"default.nnw"                 +$
  " -FILTER_NAME "    +sex_dir+filter+".conv"                +$
  " -PARAMETERS_NAME "+sex_dir+paramfile                     +$
  " -CATALOG_TYPE "   +format                                +$
  " -CATALOG_NAME "   +output_file+"."+extension
if keyword_set(segmentation) or keyword_set(background_level) or keyword_set(rms_background) then begin
  checkimage_type=" -CHECKIMAGE_TYPE "
  checkimage_name=" -CHECKIMAGE_NAME "
endif else begin
 checkimage_type=" "
 checkimage_name=" "
endelse
if keyword_set(segmentation) then begin
  checkimage_type=checkimage_type+'"SEGMENTATION",'
  checkimage_name=checkimage_name+'"'+output_file+'_seg.fits",'
endif
if keyword_set(background_level) then begin
  checkimage_type=checkimage_type+'"-OBJECTS",'
  checkimage_name=checkimage_name+'"'+output_file+'_back.fits",'
endif
if keyword_set(rms_background) then begin
  checkimage_type=checkimage_type+'"BACKGROUND_RMS",'
  checkimage_name=checkimage_name+'"'+output_file+'_rms.fits",'
endif
checkimage_type=strmid(checkimage_type,0,strlen(checkimage_type)-1)
checkimage_name=strmid(checkimage_name,0,strlen(checkimage_name)-1)
command_line=command_line+checkimage_type+" "+checkimage_name
if keyword_set(silent) then $
  command_line=command_line+" -VERBOSE_TYPE QUIET "
if keyword_set(filename_detect) then begin
if ( file_test(input_file+"_weight.fits") ) and ( file_test(input_file_detect+"_weight.fits") ) then begin
    command_line = command_line      +$
      " -WEIGHT_TYPE MAP_WEIGHT,MAP_WEIGHT "      +$
      " -WEIGHT_IMAGE "+input_file_detect+"_weight.fits,"+input_file+"_weight.fits"
endif else if ( file_test(input_file_detect+"_weight.fits") ) then $
  command_line=command_line         +$
  " -WEIGHT_TYPE MAP_WEIGHT,NONE "   +$
  " -WEIGHT_IMAGE "+input_file_detect+"_weight.fits,"+input_file_detect+"_weight.fits" $
else if ( file_test(input_file+"_weight.fits") ) then $
  command_line=command_line             +$
  " -WEIGHT_TYPE NONE,MAP_WEIGHT "           +$  ;MF - dual image mode weighting
  " -WEIGHT_IMAGE "+input_file+"_weight.fits,"+input_file+"_weight.fits"
endif else if ( file_test(input_file+"_weight.fits") ) then $
  command_line=command_line          +$
  " -WEIGHT_TYPE MAP_WEIGHT "          +$
  " -WEIGHT_IMAGE "+input_file+"_weight.fits" 
if keyword_set(seeing) then begin
    message,'using input SEEING ='+strtrim(seeing),/continue
    command_line=command_line        +$
  " -SEEING_FWHM "+strtrim(string(seeing),2)+" "
endif


;MF - other stuff to mess with:
if keyword_set(detect_minarea) then command_line=command_line       +$
  " -DETECT_MINAREA "+strtrim(string(detect_minarea),2)+" "

if keyword_set(detect_thresh) then command_line=command_line          +$
  " -DETECT_THRESH "+strtrim(string(detect_thresh),2)+" "

if keyword_set(analysis_thresh) then command_line=command_line         +$
  " -ANALYSIS_THRESH "+strtrim(string(analysis_thresh),2)+" "

if keyword_set(gain) then command_line=command_line            +$
  " -GAIN "+strtrim(string(gain),2)+" "

if keyword_set(deblend_nthresh) then command_line=command_line      +$
  " -DEBLEND_NTHRESH "+strtrim(string(deblend_nthresh),2)+" "

if keyword_set(deblend_mincont) then command_line=command_line     +$
  " -DEBLEND_MINCONT "+strtrim(string(deblend_mincont),2)+" "

if keyword_set(memory_objstack) then command_line=command_line     +$
  " -MEMORY_OBJSTACK "+strtrim(string(memory_objstack),2)+" "

if keyword_set(memory_pixstack) then command_line=command_line     +$
  " -MEMORY_PIXSTACK "+strtrim(string(memory_pixstack),2)+" "    

if keyword_set(memory_bufsize) then command_line=command_line      +$
  " -MEMORY_BUFSIZE "+strtrim(string(memory_bufsize),2)+" "


; Run SExtractor
spawn,command_line,exit_status=exit_status


;; Add some useful information to the header
;if not keyword_set(ASCII) then begin
;  catalogue = mrdfits(file+"."+extension, 2, header, /SILENT)  
;  paramfile='shex.param'
;  param_file_string="PARAMFL = '"+paramfile+"'"
;  param_file_string=param_file_string+strmid(header[n_elements(header)-1],strlen(param_file_string))
;  insert_here=where(strupcase(strmid(header,0,7)) eq "EXTNAME",insert)
;  if insert gt 0 then header=[header[0:insert_here-1],param_file_string,header[insert_here:*]]
;  ; This next line doesn't write it out in the correct format!
;  mwrfits, catalogue, file+"b."+extension, header
;endif


; Report success or failure
if exit_status eq 0 then begin
  message,"SExtractor catalogue written to "+output_file+"."+extension,/info,noprint=silent
endif else begin
  message,"SExtractor failed to run"
endelse


; Read SExtractor catalogue into memory if desired
if arg_present(sexcat) then simage_shapelets_read_sexcat,sexcat,filename,     $
                                                  full_path=full_path, $
                                                  silent=silent


end
