;+
; NAME:
; simage
;
; CALLING SEQUENCE 
; simage, param_file, keyword=keyword
;
; PURPOSE:
; This will make a simulated image based on the HST Ultra Seep Field images
; The UDF bands are B V i z J H
; It uses a shapelet catalog which has been contructed from the UDF
; image galaxies
;
; INPUTS:
; param_file - a ascii file that supplies all of the parameters needed
;              by the program. This parameter file is read line by
;              line and each line is executed exactly as written. This
;              sets the parameters needed by the program. These
;              parameters can be over-written by adding the keyword to
;              this program and editing the parameter file to start
;              with "if n_elements(keyword) eq 0 then ...". This works
;              because the parameter file is executed after reading
;              the keywords.
;
; DESCIPTION OF PARAMETERS:
; throughput_ratio - (6 array) These are the adjustments to the throughput
;                    relative to the UDF. This is one way of allowing
;                    for changes in the filter curves. This throughput
;                    does NOT include the change in collecting area of
;                    the mirror. If the parameter "filter_files" is
;                    specified, then "throughput_ratios" are
;                    recalculated and so this parameter is ignored.
;                    See code below to see how throughput_ratios are calculated 
;                    for any filter.                       
; pixel_scale - (6-array) The pixel scale in arc-seconds/pixel for each filter.
; psf_type - May be "snap", "dune","udf" or "Gaussian". The size can
;            be changed with parameter ee50.
; collecting_area - the collecting area of the telescope in
;                   m^2. Given by Pi *(R_max^2-R_min^2) for a typical
;                   annular telescope mirror.
; n_begin, n_end  - the filters to begin and end on
; exposure_time - exposure tim in seconds
; area - area of field of view in square arcminutes
; random_seed - the random seed to use for sampling galaxies and
;               noise. If set to "RANDOM", this will be chosen from
;               the system time.
; gamma - the shear [gamma_1,gamma_2]
; outfile - the output file name string
; n_star - the number of stars to add (must be at least 1)
; n_gal - number of galaxies to add. If set to "DEFAULT",
;         this will be chosen based on numbers tuned to the
;         COSMOS survey number counts.
; ee50 - the encircled energy of the PSF in arcseconds. If set to
;        "DEFAULT", the specified psf will be used unaltered.
; OUTPUTS:
; writes images to the subdirectory "data" of the directory
; specified by environment variable $shapelet_sims_dir
;
; REVISION HISTORY:
; Written by David Johnston (Caltech/JPL) Jul 10, 2008
; Based on a previous version udf_snap_sim.pro by Matt Ferry
; Based on udf_simulate, This one uses an input galaxy number counts
; relation of the Form n(m) = B 10^(A*m) , David Johnston Mar 18, 2009
;-

pro simage,param_file,$
throughput_ratio=throughput_ratio,pixel_scale=pixel_scale,psf_type=psf_type,collecting_area=collecting_area,n_begin=n_begin,$
n_end=n_end,exposure_time=exposure_time,area=area,random_seed=random_seed,gamma=gamma,outfile=outfile,$
n_star=n_star,n_gals=n_gals,ee50=ee50,debug=debug, noise_seed = noise_seed, psf_name = psf_name, $ 
fwhm_input=fwhm_input, width=width

;Read in the parameters from param_file, keywords over-ride the
;param_file parameters

;logfile=shapelets_paths(3,/silent)+outfile[0]+'.log'


;if 1 then begin
;   ;write to a logfile
;   get_lun,lunit
;   openw,lunit,logfile
;endif else begin
;   ;just write to screen
;   lunit=-1
;endelse

print,'Reading param_file: ',param_file

get_lun,unit
openr,unit,param_file
while not eof(unit) do begin
    line=''
    readf,unit,line
    char0=strmid(strcompress(line,/rem),0,1)
    ;if char0 ne ';' then print,line
    junk=execute(line)
endwhile
free_lun,unit

print,"throughput_ratio = ",throughput_ratio
print,"pixel_scale = ",pixel_scale
print,"psf_type = ",psf_type
print,"collecting_area = ",collecting_area
print,"n_begin = ",n_begin
print,"n_end = ",n_end
print,"exposure_time = ",exposure_time
print,"area = ",area
print,"random_seed = ",random_seed
print,"gamma = ",gamma
print,"outfile = ",outfile
print,"n_star = ",n_star
print,"n_gals = ",n_gals
print,"ee50 = ",ee50

;n_star must be at least 1. No stars will be added.
;if n_star eq 0 then begin
;n_star = 1
;endif

;flush,lunit

if keyword_set(debug) then stop

if n_elements(filter_files) ne 0 then begin
    udf_fils=['B','V','I','Z','J','H']
    if filter_files[0] ne "NONE" then begin
        fex=0
        for i=0, 5 do fex=fex+file_test(filter_files[i])
        if fex ne 6 then begin
            print,'Error - cannot find all filter_files'
            print,filter_files
            print,'using input throughput_ratios instead'
        endif else begin
            ;all exist
            for i=0, 5 do begin
                readcol,filter_files[i],lambda,Trans
                ;calculate the filter "width"
                W=int_tabulated(lambda,Trans/lambda)
                simage_get_filter_zero,'hst',udf_fils[i],s_hst
                tratio=W/s_hst.w
                throughput_ratio[i]=tratio
            endfor
        endelse
    endif

    print,"Calculated throughput_ratio = ",throughput_ratio
endif

if keyword_set(debug) then stop

;Number_gals=long(5570)         ;This gets the Cosmos sim to agree with
                                ;Alexie's CUMMULATIVE Number counts
                                ;and also degraded UDF

slope_nc=0.332                  ;Frome Leauthaud et al , COSMOS
amp_nc=0.000572836              

;adjustments for the fact that input not equal to output numbercounts
;due to various reasons
;this was calibrated by David Johnston on Mar 18,2009

if n_elements(amp_adjust) eq 0 then amp_adjust = 0.89576536
slope_adjust=0.0

amp_nc=amp_nc*amp_adjust
slope_nc=slope_nc+slope_adjust

seed=random_seed
simage_rot=1                ;do the object rotations or not
if simage_rot then rotseed_start=seed

if n_elements(exposure_time) eq 1 then exposure_time=replicate(exposure_time,6)

path = simage_shapelets_paths(3,/silent)
restore,path+'shapecat_total_trim.sav'
;Temporary - cut rsquared lt 50, mag lt -3
band = ['b','v','i','z','j','h']

;if n_gal eq "DEFAULT" then n_gal = long(Number_gals*(area/11.49)) 
if n_elements(n_star) eq 0 then n_star = long(100*(area/11.49)) 
if n_elements(drizzle_fac) eq 0 then drizzle_fac=1.0

collecting_area_hst=4.52389     ; pi*1.2^2
throughput = (collecting_area/collecting_area_hst)*throughput_ratio

;Pixel scale for the catalogue
pix_scale_catalogue=fltarr(6)+0.03
pix_scale_catalogue[4:5]=[0.09,0.09]

;Pixel size

pixsize=pixel_scale

;Number of pixels at pixel scale 0.1"/pixel 
pixels = 2034*sqrt(area/11.49)
n_pix = [pixels,pixels]

;read the psf

psf_types=['snap','dune','udf','Gaussian','none']
wpsf=where(psf_types eq psf_type,npsf)
if npsf eq 0 then begin
    print,'ERROR - psf_type must be "snap", "dune" or "udf"'
    return
endif

if psf_type eq "snap" then begin
    print,'SNAP PSF'
    simage_get_snap_psf,psf, psf_name = psf_name
    ;ignore wavelength dependence for now
endif

if psf_type eq "udf" then begin
    print,'UDF PSF'
    simage_udf_psf,psf,/read,band=n_begin
    psf.n_max=7
endif

if psf_type eq "dune" then begin
    print,'DUNE PSF'
    mirror_size=sqrt(collecting_area/!pi) ;mirror size in m
    simage_get_dune_psf,psf,pix_scale=pixsize[n_begin],mirror_size=mirror_size, psf_name=psf_name
endif

if psf_type eq "Gaussian" then begin
    print,'Gaussian PSF'
    FWHM=0.1865
    simage_shapelets_read_psf,psf,FWHM
endif

;select the width of the PSF via either EE50 or FWHM
;Note EE50 and FWHM should be in arcseconds

if width eq 'EE50' then begin
	print, 'EE50 input value being used for PSF width'
	if n_elements(ee50) eq 1 then ee50=replicate(ee50,6)
	simage_get_ee50,psf,ee50_pix
	ee50_desired_pix=ee50[n_begin]/pixsize[n_begin]
	psf.beta=psf.beta*ee50_desired_pix/ee50_pix
	simage_get_ee50,psf,ee50_pix_new
	print,"ee50_desired",ee50_desired_pix," pixels"
	print,"ee50_new",ee50_pix_new," pixels"

endif 

if width eq 'FWHM' then begin
	print, 'FWHM input value being used for PSF width'
	if n_elements(fwhm_input) eq 1 then fwhm_input=replicate(fwhm_input,6)
        simage_get_ee50,psf,fwhm=fwhm
	fwhm_pix = fwhm
	fwhm_desired_pix = fwhm_input[n_begin]/pixsize[n_begin]
	print, 'fwhm_pix=', fwhm_pix, " pixels"
	print, 'psf.beta =', psf.beta," pixels"
	psf.beta=psf.beta*fwhm_desired_pix/fwhm_pix
        simage_get_ee50,psf,fwhm=fwhm 
        print,"new fwhm",fwhm," pixels"  
        print, 'fwhm_pix = ', fwhm_pix, " pixels"
        print, 'fwhm_desired_pix = ', fwhm_desired_pix, " pixels" 
        print, 'psf.beta = ', psf.beta, " pixels"
endif



;normalize the PSF
psf_tot=simage_shapelets_flux(psf)
psf.coeffs=psf.coeffs/psf_tot
psf.coeffs_error=psf.coeffs_error/psf_tot

if keyword_set(debug) then stop

;Assemble the image, add noise

out=outfile

simage_assemble_image,new_shapecat[n_begin],out[n_begin],n_pix=long(n_pix*0.1/pixsize[n_begin]),$
dumpfile=dumpfile,pos_array=pos_array,red_arr=red_arr,pix_scale_catalogue=pix_scale_catalogue[n_begin],$
pixsize=pixsize[n_begin],n_star=n_star,psf=psf,star_nstart=n_star,star_pos_array=star_pos_array,$
zeropoint=zeropoint[n_begin],rotseed=rotseed_start,seed=seed,gamma=gamma, objlist=objlist,adjustflux=adjustflux,$
slope_nc=slope_nc,amp_nc=amp_nc,sky_area=area,debug=debug,read_noise=read_noise, n_gals = n_gals

if keyword_set(debug) then stop

pos_array = pos_array*pixsize[0]/pixsize[n_begin]
if n_elements(star_pos_array) ne 0 then star_pos_array = star_pos_array*pixsize[0]/pixsize[n_begin]

simage_add_noise,out[n_begin],exposure_time=exposure_time[n_begin],throughput=throughput[n_begin],filename_output=out[n_begin]+'_noisy',/weight,$
  /update,sky_level=sky_level[n_begin],positive_definite=positive_definite,seed=seed, background = background, noise_seed = noise_seed

if keyword_set(debug) then stop

;Repeat
for i=n_begin+1,n_end do begin
    
    if psf_type eq "udf" then begin
        simage_udf_psf,psf,/read,band=i
    endif

if width eq 'EE50' then begin
	print, 'EE50 input value being used for PSF width'
	simage_get_ee50,psf,ee50_pix
	ee50_desired_pix=ee50[n_begin]/pixsize[n_begin]
	psf.beta=psf.beta*ee50_desired_pix/ee50_pix
	simage_get_ee50,psf,ee50_pix_new
	print,"ee50_desired",ee50_desired_pix," pixels"
	print,"ee50_new",ee50_pix_new," pixels"

endif 

if width eq 'FWHM' then begin
	print, 'FWHM input value being used for PSF width'
	simage_get_ee50,psf,fwhm=fwhm
	fwhm_desired_pix = fwhm_input[n_begin]/pixsize[n_begin]
	fwhm_pix = fwhm_imput
	print, 'fwhm_pix=', fwhm_pix, " pixels"
	print, 'psf.beta =', psf.beta," pixels"
	psf.beta=psf.beta*fwhm_desired_pix/fwhm_pix
endif

    simage_get_ee50,psf,ee50_pix
    simage_ee50_desired=ee50[i]/pixsize[i]
    psf.beta=psf.beta*ee50_desired/ee50_pix
    simage_get_ee50,psf,ee50_pix_new
    print,"ee50_desired",ee50_desired," pixels"
    print,"ee50_new",ee50_pix_new," pixels"

    ;normalize the PSF                                                                                                                
    psf_tot=simage_shapelets_flux(psf)
    psf.coeffs=psf.coeffs/psf_tot
    psf.coeffs_error=psf.coeffs_error/psf_tot

    simage_assemble_image,new_shapecat[i],out[i],n_pix=long(n_pix*0.1/pixsize[i]),$
      pos_array=pos_array*pixsize[0]/pixsize[i],red_arr=red_arr,pix_scale_catalogue=pix_scale_catalogue[i],$
      pixsize=pixsize[i],n_star=n_star,psf=psf,star_pos_array=star_pos_array*pixsize[0]/pixsize[i],$
      zeropoint=zeropoint[i],rotseed=rotseed_start,seed=seed, objlist=objlist,adjustflux=adjustflux,$
      slope_nc=slope_nc,amp_nc=amp_nc,sky_area=area,read_noise=read_noise, n_gals = n_gals

    simage_add_noise,out[i],exposure_time=exposure_time[i],throughput=throughput[i],filename_output=out[i]+'_noisy',/weight,$
      /update,sky_level=sky_level[i],positive_definite=positive_definite,seed=seed, background = background, noise_seed = noise_seed

    if keyword_set(debug) then stop
endfor 

;if lunit ne -1 then free_lun,lunit

if keyword_set(debug) then stop

return
end
