pro simage_add_noise, filename,                              $
                      IMAGE             = image,             $
                      EXPOSURE_TIME     = exposure_time,     $
                      FILENAME_OUTPUT   = filename_output,   $
                      FULL_PATH         = full_path,         $
                      SEED              = seed,              $
                      THROUGHPUT        = throughput,        $
                      DARK_CURRENT      = dark_current,      $
                      BIAS              = bias,              $
                      READ_NOISE        = read_noise,        $
                      WEIGHT_MAP        = weight_map,        $
                      SKY_LEVEL         = sky_level,         $
                      SKY_SMOOTH        = sky_smooth,        $
                      SKY_KERNEL        = sky_kernel,        $
                      POSITIVE_DEFINITE = positive_definite, $
                      BACKGROUND        = background,        $ 
                      SATURATION_LEVEL  = saturation_level,  $
                      UPDATE            = update,            $
                      CCDWIDTH          = ccdwidth,          $
                      CCDTHICKNESS      = ccdthickness,      $
                      CRDIFFUSION       = crdiffusion,       $
                      CRFLUX            = crflux,            $
                      CRN_ELEC          = crn_elec,          $
                      CRSIGMA           = crsigma,           $
                      HIDE_SHEAR        = hide_shear,        $
                      noise_seed = noise_seed
                              
 
;$Id: simage_add_noise.pro,v 1.5 2006/04/21 21:39:36 fwh Exp $
;
; Copyright ? 2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SIM_ADDNOISE
;
; PURPOSE:
;       Adds a model of observational noise to a simulated image.
;
; CATEGORY:
;       Astronomical image simulation.
;
; INPUTS: 
;          FILENAME     - Name of input image, with path but w/o extension.
;	         EXPOSURE_TIME- [seconds] Exposure time.
;
; OPTIONAL INPUTS:
;          IMAGE        - Image structure, to save time loading it from disc.
;          FILENAME_OUT - Output filename. Can also supply an undefined variable
;                         so that the new file name is returned in it.
;          THROUGHPUT   - Camera throughput relative to that used to take the
;                         image from which the galaxy morphology PDF is drawn.
;                         This should include CCD effects like quantum efficiency,
;                         filter effects like the width and camera optical effects.
;          SEED         - Random number seed.
;          SKY_LEVEL    - [counts/second/arcsec^2] sky background level. This
;                         is only added if the BACKGROUND flag is also set, but
;                         it is always used to calculate the rms sky noise level.
;          SKY_KERNEL   - [pixels] sky background is convolved with a kernel
;                         of this size, to simulate DRIZZLE and seeing. For this
;                         to be used, the SKY_SMOOTH flag must also be set.
;          READ_NOISE   - [counts] Rms of read noise to be added.
;          BIAS         - [counts] Bias value to be added without noise.
;          SATURATION_L - [counts] Saturation level of CCD, before background
;                         or bias level subtracion.
;          COSMIC_RAYS  - Add cosmic rays, using model by Jodi Lamoureux.
;          CCDWIDTH     - [microns] CCD width.
;          CCDTHICKNESS - [microns] CCD thickness.
;          CRDIFFUSION  - Cosmic rays diffusion? 0=off, 1=on.
;          CRFLUX       - Cosmic rays per second per cm^2.                                                 
;          CRN_ELEC     - [e-] Electrons emitted per micron of silicon.                                        
;          CRSIGMA      - [microns] rms diffusion for 200 micron drift.                                        
;
; KEYWORD PARAMETERS:
;          UPDATE       - First update a noise-free image created with v2.x of
;                         simage software? This option is only to maintain
;                         backwards compatibility.
;                         0=no, 1=yes.
;          FULL_PATH    - As in other shapelet routines, are the file names
;                         relative to paths defined in shapelets_paths?
;                         0=relative, 1=complete paths are used.
;          WEIGHT_MAP   - Compute an inverse variance map and write it to disc?
;                         0=no, 1=yes.
;          SKY_SMOOTH   - Enable correlated background noise?
;                         0=not at all, 1=top hat kernel, 2=Gaussian kernel.
;          BACKGROUND   - Include nonzero sky background?
;                         0=background subtracted, 1=background added.
;          POSITIVE_DEF - Apply a positive definite constraint?
;                         0=no, 1=yes, 2=absolutely.
;          HIDE_SHEAR   - Hide input shear values?
;                         0=no, 1=yes.
;
; OUTPUTS:
;       An image with added noise, and optionally an inverse variance weight
;       map, are written to disc.
;       The updated image is also returned in the "image" strucutre variable.
;
; MODIFICATION HISTORY:
;       Jan 06 - Complete rewrite from scratch by RM.
;       Jan 05 - OUTPUT_FILE keyword added by RM.
;       Aug 04 - Smoothing of noise by non-integer pixel sizes allowed by RM.
;       Jun 04 - Weight maps changed to suffix *_weight.fits by RM.
;       Oct 03 - Jodi Lamoureux's cosmic ray model incorporated.
;       Sep 03 - Inverse variance map output created by RM.
;       May 03 - Default behaviour for correlated bground noise 
;           	  changed to unsmoothed by RM.
;       Mar 03 - Program name changed to protect the innocent.
;       Jul 02 - Exposure time glitch corrected by RM.
;       Feb 02 - add_noise.pro written by Richard Massey.
;-

COMPILE_OPT idl2


;
; Parse input requirements
;
if not keyword_set(throughput)      then throughput      = 1.      ; Camera throughput relative to original source of galaxies
if not keyword_set(exposure_time)   then exposure_time   = 100000. ; [seconds] exposure time of image
if not keyword_set(sky_kernel)      then kernel          = 2.0     ; [pixels] sky noise can be colvolved with a kernel of this size 
if not keyword_set(sky_level)       then sky_level       = 0.0766 ; [counts/s/arcsec^2] brightness of sky background, estimated from HDF
if not keyword_set(saturation)      then saturation      = 4096.0  ; [counts] saturation level
if not keyword_set(CCDwidth)        then CCDwidth        = 10.5    ; [microns] CCD width
if not keyword_set(CCDthickness)    then CCDthickness    = 200.0   ; [microns] CCD thickness
if not keyword_set(CRdiffusion)     then CRdiffusion     = 0       ; Cosmic rays diffusion 0=off, 1=on
if not keyword_set(CRflux)          then CRflux          = 4.5     ; Cosmic rays per second per cm^2
if not keyword_set(CRn_elec)        then CRn_elec        = 70.0    ; Electrons emitted per micron of silicon
if not keyword_set(CRsigma)         then CRsigma         = 3.5     ; [microns] lateral diffusion for 200 micron drift
if not keyword_set(filename_output) then filename_output = filename+"_t"+strmid(strtrim(exposure_time,1),0,5)

;
; Read in image
;
if not keyword_set(image) then $
simage_shapelets_read_image,image,filename,FULL_PATH=full_path,SILENT=silent,$
                           /NO_NOISE,/NO_SEG,/NO_MASK
header=image.header
sxaddpar,header,"EXPTIME",exposure_time," seconds"
sxaddpar,header,"NOISEMOD",0B," Parameters of applied noise model"


;
; Update noise-free simulated images created with v2.x of software, and
; thereby maintain backwards compatibility
;
if keyword_set(update) then begin
  image.units="cps"
  sxaddpar,header,"BUNIT",image.units
  sxaddpar,header,"ORIGINAL","HDF-N and HDF-S"," Original source of galaxy morphologies",AFTER="EXPTIME"
endif


;
; Convert image units to total number of counts
;
case image.units of
  "cps": image.image=image.image*exposure_time
  "counts":
  else: message,'Image units "'+image.units+'" are not recognised!',/INFO
endcase
image.exposure_time=exposure_time
sxaddpar,header,"EXPTIME",image.exposure_time


;
; Normalise counts to those in telescopes with a higher throughput (including
; quantum efficiency, filter width etc) than that of the original image
;
image.image=image.image*throughput
sxaddpar,header,"THROPUT",throughput," Relative throughput of this camera",AFTER="ORIGINAL"
image.photo_zp=image.photo_zp+2.5*alog10(throughput)
junk=sxpar(header,"PHOTOZP",count=count)
if count eq 1 then begin
  sxaddpar,header,"PHOTOZP",image.photo_zp," Photometric zeropoint"
endif else begin
  sxaddpar,header,"PHOTZP",image.photo_zp," Photometric zeropoint"
endelse


;
; Apply positive-definite constraint
;
; The first option imposes a positive-definite constraint, but conserves
; clumpiness. The second was used for Massey et al. (2004). Both will affect
; shear.
if keyword_set(positive_definite) then begin 
  message,"Imposing positive-definite constraint",/INFO
  case positive_definite of
       1: image.image = abs(temporary(image.image))
    else: image.image = temporary(image.image) > 0.
  endcase
  sxaddpar,header,"NOISEPOS",1B," Positive-definite constraint #"+strtrim(positive_definite,2)+" imposed",AFTER="NOISEMOD"
endif else sxaddpar,header,"NOISEPOS",0B," Positive-definite constraint not imposed",AFTER="NOISEMOD"


;
; Add cosmic rays (NB: should also have added these to the inverse variance map)
;
if keyword_set(cosmic_rays) then begin
  message,"Superimposing cosmic rays",/INFO
  cnoise = fltarr(im_size[0],im_size[1])
  dummy  = call_external(shapelets_paths(0,/silent)+"simages/cosmic_rays/CosmicRay.so",$
           "cosmicRayIDL", cnoise, im_size[0], im_size[1], $
           CCDwidth, CCDthickness, exposure_time,          $
           CRdiffusion, CRflux, CRn_elec, CRsigma)
  image  = temporary(image)+cnoise/exposure_time
  delvarx,cnoise
  sxaddpar,header,"NOISECR",1B," Cosmic rays added",AFTER="NOISEMOD"
endif else sxaddpar,header,"NOISECR",0B," No cosmic rays added",AFTER="NOISEMOD"


;
; Work out sky noise properties
;
sky_level_per_pixel=sky_level*exposure_time*throughput*image.pixel_scale^2
sky_rms_per_pixel=sqrt(sky_level_per_pixel)

;
; Construct inverse variance weight map
;
if keyword_set(weight_map) then begin
  message,"Constructing inverse variance weight map",/INFO
  image_noise=float(exposure_time^2/(image.image>0+float(sky_rms_per_pixel)^2))
  if tag_exist(image,"noise") then begin
    image.noise=temporary(image_noise)
  endif else begin
    image=create_struct(image,"noise",temporary(image_noise))
  endelse
endif

seed = noise_seed

;
; Add photon counting noise
;
message,"Adding photon counting noise",/INFO
image.image=temporary(image.image)+sqrt(image.image>0)*$
            randomn(seed,image.n_pixels[0],image.n_pixels[1])
sxaddpar,header,"NOISEPHT",1/sqrt(exposure_time)," Photon counting shot noise rms per pixel value",AFTER="NOISEMOD"
;
; *** Slow but memory-efficient method ***
;for y=0,im_size(2)-1 do for x=0,im_size(1)-1 do $
; image[x,y] = image[x,y]+sqrt(image(x,y))*randomn(seed)
; Getting y,x the right way round speeds it up a bit (see RSI website)


;
; Add sky background noise
;
message,"Adding sky background noise",/INFO
sky_noise=randomn(seed,image.n_pixels[0],image.n_pixels[1])*sky_rms_per_pixel
if keyword_set(sky_smooth) then begin
  if not keyword_set(sky_kernel) then message,"Specify size of smoothing kernel via SKY_KERNEL option"
  message,"Smoothing sky background to mimic DRIZZLE/unresolved sources",/INFO
  case sky_smooth of
    1: sky_noise=smooth(sky_noise, sky_kernel,/edge) ; Quick, top-hat kernel
    else: begin                                      ; FWHM of a Gaussian, in pixels
            smooth_gaussian, sky_noise, sky_kernel
            message,"Renormalising Gaussian kernel smoothing",/INFO
            sky_noise=sky_noise*3.4
          end
  endcase
  sxaddpar,header,"NOISECOR",sky_kernel," pixels Smoothing length of background noise",AFTER="NOISEMOD"
endif
sxaddpar,header,"NOISESKY",sky_rms_per_pixel/exposure_time," Sky background rms",AFTER="NOISEMOD"
image.image=temporary(image.image)+sky_noise
delvarx,sky_noise


;
; Add constant sky background level (and bias)
;
if keyword_set(background) then begin
  message,"Adding sky background level",/INFO
  image.image=temporary(image.image)+sky_level_per_pixel
  if keyword_set(bias) then begin
    message,"Adding bias level",/INFO
    image.image=temporary(image.image)+bias
  endif
endif else sky_level_per_pixel=0.
sxaddpar,header,"NOISELEV",sky_level_per_pixel/exposure_time," Sky background level",AFTER="NOISEMOD"


;
; Add read noise
;
if keyword_set(read_noise) then begin
  message,"Adding read noise",/INFO
  image.image=temporary(image.image)+randomn(seed,image.n_pixels[0],image.n_pixels[1])*read_noise
  sxaddpar,header,"NOISEREA",read_noise," Read noise rms",AFTER="NOISEMOD"
  print,"Just Added read_noise =",read_noise
endif else sxaddpar,header,"NOISEREA",0B," No read noise",AFTER="NOISEMOD"


;
; Add dark current
;
if keyword_set(dark_current) then begin
  message,"Adding dark current",/INFO
  image.image=temporary(image.image)+dark_current*exposure_time
  sxaddpar,header,"NOISEDAR",dark_current," Dark current",AFTER="NOISEMOD"
endif else sxaddpar,header,"NOISEDAR",0B," No dark current",AFTER="NOISEMOD"


;
; Impose CCD saturation
;
if keyword_set(saturation_level) then begin
  image.image=image.image<saturation_level
  sxaddpar,header,"NOISESAT",saturation_level," CCD saturation limit",AFTER="NOISEMOD"
endif else sxaddpar,header,"NOISESAT",0B," No CCD saturation",AFTER="NOISEMOD"


;
; Renormalise image units to counts per second
;
image.image=temporary(image.image)/exposure_time
sxaddpar,header,"BUNIT","cps"," Image units",AFTER="NAXIS2"


;
; Update image header
;
sxaddpar,header,"DATAMAX",max(image.image,min=datamin)
sxaddpar,header,"DATAMIN",datamin
if keyword_set(hide_shear) then begin
  junk=sxpar(header,"GAMMAIN1",count=count) & if count eq 1 then sxaddpar,header,"GAMMAIN1","Hidden",/SAVECOMMENT
  junk=sxpar(header,"GAMMAIN2",count=count) & if count eq 1 then sxaddpar,header,"GAMMAIN2","Hidden",/SAVECOMMENT
endif
temp_image=create_struct("name",image.name,"type",image.type)
for i=0,n_tags(image)-1 do begin
  case strupcase((tag_names(image))[i]) of
    "NAME":
    "TYPE":
    "HEADER": temp_image=create_struct(temp_image,"header",header)
    else:     temp_image=create_struct(temp_image,(tag_names(image))[i],image.(i))
  endcase
endfor
image=temporary(temp_image)


;
; Write image to disc
;
simage_shapelets_write,image,filename_output,/IMAGE,NOISE=weight_map,FULL_PATH=full_path

end
