pro simage_generate_shapecat, shapecat, seed=seed, output_file=output_file, width=width, $
                     n_pix=n_pix, n_max=n_max, n_obj=n_obj, noperturb=noperturb, plot=plot

;$Id: simage_generate_shapecat.pro,v 1.1.1.1 2005/04/08 20:51:58 fwh Exp $
;
; Copyright ? 2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SIMAGE_GENERATE_SHAPECAT
;
; CATEGORY:
;      Shapelets image simulation.
;
; PURPOSE:
;      Uses simage_resample_pdf.pro many times to generate a new random 
;      catalogue of objects that could be used to populate a simulated image.
;
; INPUTS:
;      shapecat - Shapelet catalogue of real data in a similar regime to the
;                 required simulation (eg HDF galaxies, read in using
;                 shapelets_read_shapecat_hdf.pro).
;                 It is recommended, although not necessary, that this contain
;                 polar rather than Cartesian shapelet coefficients. Objects
;                 just morph better in polar shapelets.
;
; OPTIONAL INPUTS:
;      output_file - Filename for output.
;      width       - This is slow to calculate, so allow it to be entered by
;                    hand, or calculated on a previous run to avoid the
;                    duplication of effort.
;      seed        - Seed for random number generator.
;      n_pix       - Integer (x,y). Number of pixels planned for final
;                    simulated image. This routine does not create the image
;                    (for that, see simage_assemble_image.pro), but the random
;                    positions of simulated galaxies can be specified in either
;                    routine. If they are specified here, and stored in the
;                    catalogue, it will be easier to duplicate simulated image
;                    or created dithered simulateed image later.
;                    Default:
;      n_max       - Objects created up to this truncation in shapelet space.
;                    Default: the highest n_max used in the input shapecat.
;      n_obj       - Number of objects in output catalogue. This does not
;                    determine the number density of objects in the simulated
;                    image. That is done in simage_assemble_image.pro.
;                    Default: .
;
; KEYWORD PARAMETERS:
;      /PLOT   - Plot each newly generated object to screen, for debugging.
;      /SILENT - Operate silently.
;
; OUTPUTS:
;      A simulated shapelet catalogue is written to disc.
;
; MODIFICATION HISTORY:
;      Jun 04 - Storage of coefficients in shapecat structures changed to
;               a_ij rather than a_ij/a_00 by RM.
;      Mar 04 - Tidied up by RM.
;      Mar 03 - Cosmetic changes and modularisation of subroutines by RM.
;      Jan 02 - new_hdf.pro written by Richard Massey.
;-


;
; Read in a shapelet catalogue containg a training set of galaxy morphologies
;
if not keyword_set(shapecat) then shapelets_read_shapecat_hdf,shapecat,/polar,/moments
if shapecat.moments eq 0     then message,'Use shapelets_read_shapecat with /MOMENTS flag!'


;
; Set up default parameters
;
if not keyword_set(n_pix)       then n_pix       = 4096                                                      ; Number of pixels in final image is  n_pix x n_pix
if not keyword_set(n_max)       then n_max       = shapecat.maxn_max                                         ; Maximum shapelet order in new catalogue
if n_max gt shapecat.maxn_max   then message,'Can only generate new objects up to n_max='+strtrim(shapecat.maxn_max,2)+', as in input catalogue!'
if not keyword_set(seed)        then seed        = 1000L                                                     ; Set random number seed
if n_elements(seed) eq 1        then message,'Random seed='+strtrim(seed[0],2),/info                         ; Friendly messages with setup
if not keyword_set(n_obj)       then n_obj       = 10000                                                     ; number of new objects to synthesise...
if not keyword_set(output_file) then output_file = "new"+strmid(SYSTIME(0), 14, 2)+strmid(SYSTIME(0), 17, 2) ; Output file name
message,strtrim(string(n_obj),2)+' galaxies being created in '+shapelets_paths(2,silent=silent)+output_file+'.shape',/info,noprint=silent
seed_input=seed


;
; Calculate width for kernel smoothing in morphology PDF
;
if not keyword_set(width) then $
 simage_resample_pdf, shapecat, width=width, seed=seed, /SILENT;, pdf=pdf


;
; Prepare empty arrays to contain output catalogue
;
simage_shapelets_make_nvec,n_max,n1,n2,n_coeffs
x_cat=[[0.],[0.]] 
beta_cat=0.
n_max_cat=0
coeffs_cat=fltarr(1,n_coeffs)
coeffs_error_cat=fltarr(1,n_coeffs)


;
; Main loop to generate new objects
;
for i=1L,n_obj do begin


    ;
    ; Write progress report to screen
    ;
    if (i mod 50) eq 0 then $
      message,strtrim(string(i),2)+'/'+strtrim(string(n_obj),2),$
      /info,noprint=silent


    ;
    ; Create a new object
    ;
    delvarx,bs_obj
    simage_resample_pdf, shapecat, new_object, seed=seed, bs_obj=bs_obj, $
                         pdf=pdf, width=width, topology=topology, $
                         plot=plot, /SILENT

    
    ;
    ; Place object randomly around the sky
    ; (or just outside the image, to enable dithering and avoid edge effects)
    ;
    x = [randomu(seed),randomu(seed)]*(float(n_pix)*3/2)-[n_pix[0]/2.+50,50]


    ;
    ; Append object to new catalogue
    ;
    coeffs        = new_object.coeffs
    coeffs_error  = new_object.coeffs_error
    while ( n_elements(coeffs) lt n_coeffs ) do begin
      coeffs=[coeffs,0.] ; Pad with zeros 
      coeffs_error=[coeffs_error ,0.]
    endwhile
    coeffs_cat=[coeffs_cat,transpose(coeffs)]
    coeffs_error_cat=[coeffs_error_cat,transpose(coeffs_error)]
    x_cat=[x_cat,transpose(x)]
    beta_cat=[beta_cat,new_object.beta]
    n_max_cat=[n_max_cat,new_object.n_max]


    ;
    ; Write catalogue to disc (periodically and at end)
    ;
    if (i mod 1000) eq 0 or i eq n_obj then begin
      if i eq n_obj then begin
        history="Generated on "+systime(0)+" from training set "+shapecat.name
      endif else begin
        history="Generating on "+systime(0)+" from training set "+shapecat.name
      endelse
      output_shapecat={name:"Fake galaxy catalogue",         $
                       type:"shapecat",                      $
                       history:history,                      $
                       random_seed:seed_input,               $
                       n:i,                                  $
                       maxn_max:n_max,                       $
                       n_coeffs:n_coeffs,                    $
                       polar:0B,                             $
                       seeing:shapecat.seeing,               $
                       x:x_cat[1:*,*],                       $
                       beta:beta_cat[1:*],                   $
                       n_max:n_max_cat[1:*],                 $
                       coeffs:coeffs_cat[1:*,*],             $
                       coeffs_error:coeffs_error_cat[1:*,*], $
                       flag:fltarr(i,2),                     $
                       chisq:fltarr(i),                      $
                       sextractor:0B,                        $
                       moments:0B}
      simage_shapelets_write_shapecat,output_shapecat,output_file
    endif

endfor

end
