function simage_make_shapelet_object,decomp,$
                         gamma_input=gamma_input,psf=psf,inst_shear=inst_shear,$
                         rpos=rpos,seed=seed,shapeinfo=shapeinfo,rotang=rotang

;MF - line 83 - update shapelets_convolve to procedure
;DaveJ added rotang keyword
;$Id: simage_make_shapelet_object.pro, v1.1$
;
; Copyright ? 2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SIMAGE_MAKE_SHAPELET_OBJECT
;
; CATEGORY:
;      Shapelet simulated images.
;
; PURPOSE:
;      This module makes one object for the shapelet-based image simulations, using
;      a full shapelet model of its morphology. It is pixellated using various
;      input parameters.
; 
; INPUTS:
;      decomp  - Cartesian shapelet decomp structure of the resampled galaxy.
;
; OPTIONAL INPUTS:
;      gamma   - Weak lensing shear on all galaxies [gamma1,gamma2].
;      psf     - Cartesian decomp structure representing PSF. Will then do
;                convolvution automatically. Stars are just this PSF.
;      instrum - Post-smear shear due to geometric distortions in telescope.
;      seed    - Seed for random number generator.
;
; KEYWORD PARAMETERS:
;      RPOS    - Randomise positions and PAs of galaxies?
;      rotang - rotate by this angle degrees
; OUTPUTS:
;      recomp     - One (shapelet) simulated object in a postage stamp image.
;      shapeinfo  - A structure containing object properties for inclusion in the
;                   pre-noise catalogue.
; 
; MODIFICATION HISTORY:
;      Aug 04 - Minimum size of postage stamp increased to fix 1D bug by RM
;      Apr 04 - Pre-convolution shape parameters stored, rather than post-
;               convolution parameters. This is later written to the faked 
;               "input" SExtractor catalogue. RM.
;      Jun 03 - Written by Richard Massey
;-

COMPILE_OPT idl2

stop_file='~/STOP_SIMAGE'
if file_test(stop_file) then begin
                                ;this is used to stop the simage if
                                ;you have run it in the background and
                                ;wish to kill it, Just make this file

    print,'The stop_file ',stop_file,' exists so I will not run'
    print,'Please remove this file if you want to run'
    print,'exiting IDL'
    exit
endif

; Rotate object randomly if the positions are also random.
if keyword_set(rpos) then simage_shapelets_rotate,decomp,(randomu(seed)*360.)

;Or rotate it by command
if n_elements(rotang) ne 0 then begin
    simage_shapelets_rotate,decomp,rotang
endif

; Shear object (weak lensing).
if keyword_set(gamma_input) then simage_shapelets_shear,decomp,gamma_input,order=4,extend=2

; Calculate (pre-noise) shape properties for posterity
q=simage_shapelets_quadrupole(decomp,flux=flux)
if not keyword_set(flux) then flux=0     ;MF (2/26) - problem when an object isn't actually there.  Using zero for now.
rsq=(Q[0,0]+Q[1,1])/flux      
ell=complex(Q[0,0]-Q[1,1],2*Q[0,1])/(Q[0,0]+Q[1,1])
PA=atan(imaginary(ell),float(ell))/2.+!pi/2
A=sqrt(rsq*(1+abs(ell))/2.)
B=sqrt(abs(rsq*(1-abs(ell))/2.))
shapeinfo={A:A,B:B,theta:(PA*180/!pi) mod 360-90,rsq:rsq,flux:flux,mag:-2.5*alog10(flux)+22.08}

; Smear object (atmospheric + telescope PSF).

if keyword_set(psf) then simage_shapelets_convolve,decomp,psf;decomp=simage_shapelets_convolve(decomp,psf)  ;MF - updated, 8/24/06

; Shear object (instrumental distortions).
if keyword_set(inst_shear) then simage_shapelets_shear,decomp,inst_shear

; Smear object (CCD charge diffusion). 
; TO DO!


; Re-compose the object into pixels.
decomp.n_pixels=decomp.n_pixels+[10,10]
decomp.x=decomp.x+[5.,5.]
simage_shapelets_recomp, decomp, recomp

; Make sure we do not truncate the edges of the object. 
iter=0

while total(abs(recomp[*,0]))+$
      total(abs(recomp[0,*]))+$
      total(abs(recomp[decomp.n_pixels[0]-1,*]))+$
      total(abs(recomp[*,decomp.n_pixels[1]-1])) $
      gt (abs(total(recomp)/1e10)) do begin
  decomp.n_pixels=decomp.n_pixels+[10,10]
  decomp.x=decomp.x+[5.,5.]
  simage_shapelets_recomp, decomp, recomp
  iter=iter+1 
  if iter ge 10 then print,'Taking ',iter,' iterations inside make_shapelets_object',decomp.n_pixels
  ;print,iter
  ;if iter ge 10 then stop

endwhile


;print,'Decomp flux is',simage_shapelets_flux(decomp)
;print,'Recomp flux is',total(recomp)
;print

; That's all folks!
return, recomp

end
