
;**************************************************************;
;+
;*NAME:
;
;    WRITE_ASC
;
;*PURPOSE:
;
;    Creates an ASCII table file in which each column represents one
;    of the specified input vectors. Up to 10 parameters are allowed.
;
;*CALLING SEQUENCE:
;
;    WRITE_ASC,FILENAME,P1 [,p2,...,p8,/no_tabs,A_fmt=A_fmt]
;
;*PARAMETERS:
;
;    FILENAME  (REQ) (I) (0) (S)
;        Output file name. If no extension is specified, .txt is used.
;
;    P1  
;        first input vector to be converted to an ASCII file. The
;        number of elements in P1 determines the size of the ASCII table.
;
;    P2-P8  
;        input vectors to be converted to an ASCII file. Each vector will
;        be stored as a column in an ASCII table. Vectors must have the
;        same number of elements as P1 (or more). 
;
;*OPTIONAL PARAMETERS:
;    NO_TABS - (logical) - T/F switch to turn off writting tabs between
;		each output row value. (default is to insert TABS)
;    A_FMT   - (string) - Alphabetic format for output fields (default is A8).
;    SILENT  - suppresses output to the prompt
;
;*PROCEDURE:
;
;    ASC_WRITE uses the number of parameters in the procedure call
;    to determine the number of entries to write to each line in
;    the output ASCII file. Parameters are written out using the PRINTF
;    command, in the same order as they are specified in the procedure 
;    call.
;
;*EXAMPLE:
;
;    HELP,W,F,junk
;	W	flt 	array(1000)
;	F	flt	array(1000)
;	junk	int	array(2000)
;
;    WRITE_ASC,'MOD1',W,F,junk
;         - writes vectors to file MOD1.txt with three parameters per line
;         (i.e., W(i), F(i), and JUNK(I)). Please note that the junk vector 
;	  has 2000 elements. It is truncated to 1000 elements.
;
;*SUBROUTINES:
;
;    DECOMPOSE
;    PARCHECK
;
;*NOTES:
;
;    	Up to 8 parameters can be written to each row or record.
;
;    	There is currently no limit to the size of the input vectors,
;     	and there is no checking of the input vector sizes. The number
;     	of records written is determined solely from the number of elements
;     	in P1. If any other input vector is smaller than P1, an error will 
;     	occur. Vectors larger than P1 will be truncated in the output file.
;
;*RESTRICTION:
;	
;    	The default formats defined by IDL are used to determine the number
;     	of significant figures. Therefore, some numbers may be rounded-off!
;
;	Number of input parameters was limited to 8. Above 8 parameters
;	will cause the output line to be greater than 80 characters. This
;	may cause a problem reading the file back in (ie. wordprocessor
;	since each line would now take 2 separate lines due to format
;	overflow. You will need to modify the format statement in this
;	routine to allow over 8 fields.
;
;*MODIFICATION HISTORY:
;
;    written by RWT 12/28/90 
;    1/15/91 RWT use DECOMPOSE
;    1/22/91 PJL transferred to SUN/UNIX, added PARCHECK
;    6/19/91 PJL cleaned up; tested on SUN and VAX; updated prolog
;    7/01/91 RWT increase to 10 parameters
;    7/18/91 RWT increase to 15 parameters and add format keyword for
;            compatibility with IDL version 2. tested on VAX and SUN
;    8/22/91 RWT add a blank between parameters to avoid numbers running
;            together
;    9/13/91 RWT make internal parameter i a longword integer to be
;            compatible with new keyword NEL allowed in ASC_READ.
;    3/5/93	JKF/ACC		- moved to GHRS DAF, added TABs as 
;					default, and changed logic.
;    5/8/05  NTM added /SILENT keyword
;-                                            
;****************************************************************
pro writecol,filename,p1,p2,p3,p4,p5,p6,p7,p8,no_tabs=no_tabs,A_fmt=A_fmt,silent=silent
;
; check parameters
;
 on_error,2
 npar= n_params(0)
 if npar lt 2 then begin
    message,' ASC_WRITE,FILENAME,P1 [,p2,p3,p4,...p8]',/cont
    message,' Optional: /no_tabs,A_fmt=A_fmt'
 endif  ; npar

 on_error,0

 parcheck,npar,indgen(8)+2,'ASC_WRITE'
 npar = npar -1				; remove filename
 decompose,filename,d,u,n,e,v
              
 if (e eq '') then e = '.txt'    ; .txt is default extension
 filename = d + u + n + e + v
;
; open output file 
;
 openw,un,filename,/get_lun
 if not keyword_set(SILENT) then $
    message,' Creating output file: '+filename,/cont
;
; write out vectors using printf command
;

 if keyword_set( no_tabs ) then spacer= ' ' else spacer = string(9b)
 if (not (keyword_set( A_fmt))) then A_fmt = 'A12' 

 cmd= 'printf,un,format=fmt,strtrim(p1(i),2)'
 fmt= '(' + A_fmt + ')'
 if npar gt 1 then begin
	 for i=1, npar-1 do cmd = cmd + ',spacer,' + $
		'strtrim(p' + strtrim(i+1,2) + '(i)' + ',2)'
	 fmt = '(' + fmt + ',' + strtrim(npar-1,2) +  '(a1,' + A_fmt + '))'
 end

 for i=0L, n_elements(p1)-1 do begin
	status = execute( cmd )
 end

 free_lun,un
 if not keyword_set(SILENT) then $
   message,' RESULT: ' + strtrim(npar,2) + $
	' columns of '+strtrim(long(n_elements(p1)),2)+' points written',/cont

 return
 end  ; write_asc
